var assert 	= require('assert');
var https 	= require('../https');
var rest 	= require('eden-rest');
var string 	= require('eden-string');

describe('Server Test Suite', function() {
	// allow unauthorized request / for self signed certificates
	process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0" 

	var server = https()
	.setKeyFile(__dirname + '/ssl/apache.key')
	.setCertificateFile(__dirname + '/ssl/apache.crt');
	
	after(function() {
		server.disconnect();
	});
	
	describe('Basic Tests', function() {
		it('should start the server', function(done) {
			server
			
			.on('connected', function() {
				assert.equal('127.0.0.1', this.getHost());
				assert.equal(1337, this.getPort());	
				done();
			})
			.connect();
		});
		
		it('should process a 404', function(done) {
			server
			.on('request-start', function(request, response) {
				assert.equal('/test/request', request.path);
				assert.equal('object', typeof request.query);
				assert.equal('bar', request.query.foo);
			})
			.once('request-end', function(request, response) {
				assert.equal('foo=bar&zoo=choo', request.message);
			});
			
			var url = 'https://127.0.0.1:1337/test/request?foo=bar';
				
			rest(url)
			.setMethod('post')
			.setBody('foo=bar&zoo=choo')
			.getResponse(function(error, response, meta) {
				assert.equal(404, meta.response.statusCode);
				done();
			});
		});
		
		it('should process a request', function(done) {
			server
			.on('request-start', function(request, response) {
				assert.equal('/test/request', request.path);
				assert.equal('object', typeof request.query);
				assert.equal('bar', request.query.foo);
			})
			.once('request-end', function(request, response) {
				assert.equal('foo=bar&zoo=choo', request.message);
				
				response.message = 'It works';
				this.trigger('response', request, response);
			});
			
			var url = 'https://127.0.0.1:1337/test/request?foo=bar';
				
			rest(url)
			.setMethod('post')
			.setBody('foo=bar&zoo=choo')
			.getResponse(function(error, response, meta) {
				assert.equal('It works', response);
				done();
			});
		});
		
		it('should process a file', function(done) {
			server
			.on('request-start', function(request, response) {
				assert.equal('/test/request', request.path);
				assert.equal('object', typeof request.query);
				assert.equal('bar', request.query.foo);
			})
			.on('file-start', function(meta) {
				assert.equal('text', meta.key);
				assert.equal('test.txt', meta.name);
				assert.equal('text/plain', meta.mime);
			})
			.on('file-data', function(meta, data) {
				assert.equal('text', meta.key);
				assert.equal('test.txt', meta.name);
				assert.equal('text/plain', meta.mime);
				assert.equal('test123', data.toString('utf8'));
			})
			.on('file-end', function(meta) {
				assert.equal('text', meta.key);
				assert.equal('test.txt', meta.name);
				assert.equal('text/plain', meta.mime);
			})
			.on('request-end', function(request, response) {
				assert.equal('chris=hjhgj&date=2014-10-02T06:23:00Z&bio=kjhkj kjh kjh kjh', request.message);
				assert.equal(true, request.files instanceof Array);
				assert.equal(1, request.files.length);
				assert.equal('object', typeof request.files[0]);
				assert.equal('text', request.files[0].key);
				assert.equal('test.txt', request.files[0].name);
				assert.equal('text/plain', request.files[0].mime);
				
				response.message = 'It works';
				this.trigger('response', request, response);
			});
			
			var url = 'https://127.0.0.1:1337/test/request?foo=bar';
	
			var type = 'multipart/form-data; boundary=----WebKitFormBoundaryynLLVm245zLQ7V6y';
			
			var body = '------WebKitFormBoundaryynLLVm245zLQ7V6y\r\nContent-Disposition: '
			+ 'form-data; name="chris"\r\n\r\nhjhgj\r\n------WebKitFormBoundaryynLLVm245z'
			+ 'LQ7V6y\r\nContent-Disposition: form-data; name="date"\r\n\r\n2014-10-02T06'
			+ ':23:00Z\r\n------WebKitFormBoundaryynLLVm245zLQ7V6y\r\nContent-Disposition'
			+ ': form-data; name="bio"\r\n\r\nkjhkj kjh kjh kjh\r\n------WebKitFormBounda'
			+ 'ryynLLVm245zLQ7V6y\r\nContent-Disposition: form-data; name="text"; filenam'
			+ 'e="test.txt"\r\nContent-Type: text/plain\r\n\r\ntest123\r\n------WebKitFor'
			+ 'mBoundaryynLLVm245zLQ7V6y--';

			rest(url)
			.setMethod('post')
			.setHeaders('Content-Length', body.length)
			.setHeaders('Content-Type', type)
			.setBody(body)
			.getResponse(function(error, response, meta) {
				assert.equal('It works', response);
				done();
			});
		});
	});
});
